from typing import Any, Dict, List, Optional, Union, cast

class DataStorage:
    def save(self,
             data,
             metadat:Optional[Union[Dict[str, Any], List[Dict[str, Any]]]] = None):
        pass
